import 'dart:math';
import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/calculators/values.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/process/items/item.dart';
import 'package:fis_measure/process/primitives/area_abstract.dart';
import 'package:fis_measure/process/primitives/combos/urm_sr_roi_rect_combo.dart';

import 'package:fis_measure/process/primitives/location.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_rect_measure.dart';
// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

import '../calculator.dart';

class URMRectCal extends Calculator<URMRectMeasure, double> {
  URMRectCal(
    URMRectMeasure ref, {
    required this.type,
  }) : super(ref);
  final String type;
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    if (ref.feature == null) return;
    Size urmResultSize = const Size(0, 0);
    try {
      if (ref.application is! URMApplication) {
        return;
      }
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;
      //左上顶点
      final leftTopPercent =
          DPoint(p1.x < p2.x ? p1.x : p2.x, p1.y < p2.y ? p1.y : p2.y);
      //右下顶点
      final rightBottomPercent =
          DPoint(p1.x > p2.x ? p1.x : p2.x, p1.y > p2.y ? p1.y : p2.y);

      final startPoint = leftTopPercent.scale2Size(urmResultSize);
      final endPoint = rightBottomPercent.scale2Size(urmResultSize);

      String description = "URM\n Calculating...";
      updateStringValue(description);

      // switch (type) {
      //   case MeasureTypes.DensitySR:
      //   case MeasureTypes.SRRoiDensity:
      //   case MeasureTypes.Roi1:
      //   case MeasureTypes.Roi2:
      //     await getSRRoiDensityResult(urmApplication, startPoint, endPoint);
      //     break;
      //   case MeasureTypes.FractalDimSR:
      //   case MeasureTypes.SRROIFractalDim:
      //     await getSRRoiFractalDimResult(urmApplication, startPoint, endPoint);
      //     break;
      //   case MeasureTypes.SRRoiVel:
      //     await getSRRoiVelResult(urmApplication, startPoint, endPoint);
      //     break;
      //   case MeasureTypes.URMVelMeasure:
      //     await getURMVelMeasureResult(urmApplication, startPoint, endPoint);
      //     break;
      //   default:
      // }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  // //✅ URM 测量项 2
  // Future<void> getSRRoiDensityResult(
  //     URMApplication app, DPoint startPoint, DPoint endPoint) async {
  //   String description = "URM\n Measuring";
  //   try {
  //     GetSRRoiVelResult? result =
  //         await app.getSRRoiVelResult(startPoint, endPoint);
  //     if (result != null) {
  //       print("URM Measure output: ${result.output} ");
  //       final feature = ref.feature!;
  //       for (var output in ref.meta.outputs) {
  //         if ([MeasureTerms.URMDenROI, MeasureTerms.URMDenFractalDim]
  //             .contains(output.name)) {
  //           output.unit = VidUsUnit.percent;
  //           feature.updateFloatValue(output, result.output, output.unit);
  //         }
  //       }
  //     } else {
  //       throw Exception("URM Measure API error");
  //     }
  //   } catch (e) {
  //     description = " ";
  //     updateStringValue(description);
  //     return;
  //   }
  //   ref.application.updateRenderReady.emit(this, null);
  // }

  // //✅ URM 测量项 3 未验证需要组合测量项
  // Future<void> getSRRoiFractalDimResult(
  //     URMApplication app, DPoint startPoint, DPoint endPoint) async {
  //   String description = "URM\n Measuring";
  //   try {
  //     GetSRRoiFractalDimResult? result =
  //         await app.getSRRoiFractalDimResult(startPoint, endPoint);
  //     if (result != null) {
  //       print("URM Measure output: ${result.output} ");
  //       final feature = ref.feature!;
  //       for (var output in ref.meta.outputs) {
  //         if (output.name == MeasureTerms.URMDenFractalDim) {
  //           output.unit = VidUsUnit.None;
  //           feature.updateFloatValue(output, result.output, output.unit);
  //         }
  //       }
  //     } else {
  //       throw Exception("URM Measure API error");
  //     }
  //   } catch (e) {
  //     description = " ";
  //     updateStringValue(description);

  //     return;
  //   }
  //   ref.application.updateRenderReady.emit(this, null);
  // }

  // //✅ URM 测量项 7 未验证需要组合测量项
  // Future<void> getSRRoiVelResult(
  //     URMApplication app, DPoint startPoint, DPoint endPoint) async {
  //   String description = "URM\n Measuring";
  //   try {
  //     GetSRRoiVelResult? result =
  //         await app.getSRRoiVelResult(startPoint, endPoint);
  //     if (result != null) {
  //       print("URM Measure output: ${result.output} ");
  //       final feature = ref.feature!;
  //       for (var output in ref.meta.outputs) {
  //         if (output.name == MeasureTerms.SRRoiVel) {
  //           output.unit = VidUsUnit.percent;
  //           feature.updateFloatValue(output, result.output, output.unit);
  //         }
  //       }
  //     } else {
  //       throw Exception("URM Measure API error");
  //     }
  //   } catch (e) {
  //     description = " ";
  //     updateStringValue(description);

  //     return;
  //   }
  //   ref.application.updateRenderReady.emit(this, null);
  // }

  // //✅ URM 测量项 9
  // Future<void> getURMVelMeasureResult(
  //     URMApplication app, DPoint startPoint, DPoint endPoint) async {
  //   String description = "URM\n Measuring";
  //   try {
  //     GetURMVelMeasureResult? result =
  //         await app.getURMVelMeasureResult(startPoint, endPoint);
  //     if (result != null) {
  //       // TODO 全部内容展示
  //       print("URM Measure inMaxVel: ${result.inMaxVel} ");
  //       final feature = ref.feature!;
  //       for (var output in ref.meta.outputs) {
  //         if (output.name == MeasureTerms.URMVelMax) {
  //           output.unit = VidUsUnit.mms;
  //           feature.updateFloatValue(
  //               output, result.maxVel + app.urmMinVel, VidUsUnit.mms);
  //         }
  //         if (output.name == MeasureTerms.URMVelMin) {
  //           output.unit = VidUsUnit.mms;
  //           feature.updateFloatValue(
  //               output, result.minVel + app.urmMinVel, VidUsUnit.mms);
  //         }
  //         if (output.name == MeasureTerms.URMVelMean) {
  //           output.unit = VidUsUnit.mms;
  //           feature.updateFloatValue(
  //               output, result.meanVel + app.urmMinVel, VidUsUnit.mms);
  //         }
  //         if (output.name == MeasureTerms.URMVelStd) {
  //           output.unit = VidUsUnit.mms;
  //           feature.updateFloatValue(
  //               output, sqrt(result.varianceVel), VidUsUnit.mms);
  //         }
  //         if (output.name == MeasureTerms.Area) {
  //           output.unit = VidUsUnit.cm2;
  //           feature.updateFloatValue(output, result.roiArea, VidUsUnit.cm2);
  //         }
  //       }
  //     } else {
  //       throw Exception("URM Measure API error");
  //     }
  //   } catch (e) {
  //     description = " ";
  //     updateStringValue(description);

  //     return;
  //   }
  //   ref.application.updateRenderReady.emit(this, null);
  // }
}
