import 'dart:ui';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_straight_curvature_line_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';
import '../calculator.dart';

/// 自动弯曲度测量
class URMStraightCurvatureLineCal extends Calculator<StraightLine, double> {
  URMStraightCurvatureLineCal(
    StraightLine ref, {
    required this.type,
  }) : super(ref);
  final String type;
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;

      Size urmResultSize = const Size(0, 0);
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;

      await calculteURMCurvature(urmApplication, p1, p2);
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  // // ✅ URM 测量项 1
  Future<void> calculteURMCurvature(
      URMApplication app, DPoint startPoint, DPoint endPoint) async {
    String description = "URM\n Measuring";
    try {
      URMMeasureProcessResult? result = await app.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.placeHolder_0,
        srcDPoints: [
          UrmPoint(x: startPoint.x, y: startPoint.y),
          UrmPoint(x: endPoint.x, y: endPoint.y)
        ],
      );
      if (result != null) {
        print(
            "URM Measure curvature: ${result} nums: ${result.resultDPoints?.length}");
        // TODO 将返回的视图百分比区域的点集转换为全图百分比点集，然会然后绘制
        final feature = ref.feature!;
        if (feature is! StraightCurvatureFeature) return;
        feature.autoLinePoints = urmPointsToDPoints(result.resultDPoints);

        /// TODO 只转数据类型还不够，需要将 视图区域百分比坐标转为全图百分比坐标
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.SRCurvature) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.resultData, output.unit);
          }
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      description = " ";
      updateStringValue(description);
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }

  List<DPoint> urmPointsToDPoints(List<UrmPoint>? points) {
    if (points == null) return [];
    List<DPoint> urmPoints = [];
    for (var point in points) {
      urmPoints.add(DPoint(point.x, point.y));
    }
    return urmPoints;
  }
}
